import pygame

from effects import *
from graphics import *
from util import *

class ExitState(Exception):
    pass

class NewState(Exception):
    def __init__(self,ns):
        self.state = ns

divx = 4
divy = 4
class State:
    def __init__(self):
        """Initialize a state"""
        self.world = []
        self.backdrop = None
        self.screen = pygame.display.get_surface()
        self.throw = None
        self.clock = pygame.time.Clock()
        self.dt = 0
    def sort_world(self):
        def cmp_sprite(sp1,sp2):
            return cmp(sp1.priority,sp2.priority)
        self.world.sort(cmp_sprite)
        for sprite in self.world:
            self.move(sprite)
    def move(self,sprite,pos=None):
        if pos:
            sprite.pos = pos[:]
    def _beforeloop(self):
        """What to do before we go into a loop"""
        self.updatedrects = []
    def loop(self):
        """Run until we should stop"""
        self._beforeloop()
        run = True
        ret = None
        while run:
            try:
                self.iteration()
            except ExitState:
                run = False
            except NewState,e:
                run = False
                ret = e.state
        img = self.screen.convert()
        fade(img,2,mode=-1)
        return ret
    def iteration(self):
        """One iteration"""
        if self.throw:
            raise self.throw
        self.dt = self.clock.tick(60)
        self._beforerender()
        self._render()
        self._afterrender()
        self._events()
    def _beforerender(self):
        """Do stuff before we render"""
    def _render(self):
        """Update the display"""
        self.updatedrects = []
        changed = 0
        while changed < len(self.world):
            if self.world[changed].has_changed():
                break
            changed += 1
        clear = len(self.world)-1
        while clear >= changed:
            if self.world[clear].behind and \
                            self.world[clear].last_pos_drawn:
                self.screen.blit(self.world[clear].behind,
                        self.world[clear].last_pos_drawn)
                self.updatedrects.append([self.world[clear].last_pos_drawn,
                    self.world[clear].size])
            clear -= 1
        draw = changed
        while draw < len(self.world):
            self.drawnew(self.world[draw])
            draw += 1
        #pygame.display.update(self.updatedrects)
        pygame.display.flip()
    def _events(self):
        """Low level event management"""
        for e in pygame.event.get():
            if e.type==pygame.QUIT:
                raise ExitState
            else:
                self.event(e)
    def _afterrender(self):
        """Do stuff when rendering is done"""
    def event(self,e):
        """Handle some event"""
    def assign_back(self,sprite,pos,size,surf=None):
        if not surf: surf = self.screen
        sprite.behind = pygame.Surface(sprite.size).convert()
        sprite.behind.blit(surf,[0,0],[sprite.pos,sprite.size])
    def drawnew(self,sprite,surf=None):
        """Clear old spot of sprite, update sprite position
        and draw sprite in new position"""
        if not surf:
            surf = self.screen
        self.updatedrects.append([sprite.pos,sprite.size])
        self.assign_back(sprite,sprite.pos,sprite.size,surf)
        sprite.last_pos_drawn = sprite.pos[:]
        sprite.dirty = False
        for img in sprite.images:
            img.paint(surf,sprite.pos)

class MouseState(State):
    """Any state that uses a mouse"""
    def __init__(self):
        State.__init__(self)
        self.cursor = Sprite()
        self.cursor.loadimage('gui/cursor.png')
        self.cursor.priority = 500
        self.cursor.over = []
        self.world.append(self.cursor)
    def event(self,e):
        State.event(self,e)
        if e.type==pygame.MOUSEMOTION:
            self.move(self.cursor,e.pos[:])
        if e.type==pygame.MOUSEBUTTONUP:
            for b in self.cursor.over:
                if hasattr(b,"mouse_click"):
                    b.mouse_click()
    def _beforerender(self):
        nothit = []
        hit = []
        for sprite in self.world:
            if sprite == self.cursor: continue
            if point_in_rect(self.cursor.pos,sprite.pos,sprite.size):
                hit.append(sprite)
            else:
                nothit.append(sprite)
        for sprite in hit:
            if sprite not in self.cursor.over:
                self.cursor.over.append(sprite)
                if hasattr(sprite,"mouse_enter"):
                    sprite.mouse_enter()
        for sprite in nothit:
            if sprite in self.cursor.over:
                self.cursor.over.remove(sprite)
                if hasattr(sprite,"mouse_exit"):
                    sprite.mouse_exit()
        State._beforerender(self)

class GameState(MouseState):
    def quit(self):
        self.throw = ExitState()
    def start(self):
        self.throw = NewState(InGameState())

from logic import *
import random
class InGameState(GameState):
    def __init__(self):
        GameState.__init__(self)
        self.setloc(0)
        pygame.mixer.music.load(data.filepath('music/song2.ogg'))
        pygame.mixer.music.play(0)
        #oldscr = self.screen
        #newscr = self.screen = pygame.Surface([640,480])
        #self.iteration()
        #fade(newscr,3)
        #self.screen = oldscr
        #self.screen.blit(newscr,[0,0])
        self.date = 0
        self.peoples = []
        self.hq = HQ()

        self.oxygen = 100
        self.oxygendeath = 3000

        possible_people = ['bandit','carrier','civilian',
                'explorer','guard','medic','scientist']
        for i in range(35):
            people = Man(random.choice(possible_people))
            people.click_event = self.killman(people)
            self.hq.peoples.append(people)
    def killman(self,man=None):
        def f(man=man):
            if not man.alive: return
            man.set_frame(0,'dead',False)
            man.alive = False
        return f
    def _beforerender(self):
        GameState._beforerender(self)
        self.date += self.dt
        if self.date>1000:
            self.date = 0
            if self.hq.peoples:
                people = self.hq.peoples.pop(-1)
                people.pos = [100,480-20-30]
                self.peoples.append(people)
                if self.loc == people.loc:
                    self.world.insert(self.peoples_entrance,people)
        currentlocmen = [x for x in self.world if isinstance(x,Man)]
        for man in currentlocmen:
            man.add_frame(self.dt)
            if man.alive:
                self.oxygen -= self.dt/1000.0
                np = [man.pos[0]+self.dt/1000.0*man.speed*man.dir,man.pos[1]]
                if np[0]>640:
                    man.dir = -1
                if np[0]<0:
                    man.dir = 1
                self.move(man,np)
        if self.oxygen <= 0:
            self.oxygen = 0
            self.oxygendeath -= self.dt
            if random.randint(0,1000)>950:
                alivemen = [x for x in currentlocmen \
                    if x.alive]
                if alivemen:
                    people = random.choice(alivemen)
                    people.alive = False
                    people.set_frame(0,'choke',False)
                    people.choke_sound.play()
        self.oxygen_g.updatetext('Oxygen=%d'%self.oxygen)
        self.oxygen_g.loadimage('gui/button_1.png')
    def setloc(self,loc):
        self.loc = loc
        self.screen.fill([0,0,0])
        pygame.display.flip()
        self.world = []
        self.peoples_entrance = 0
        #add background
        self.sky = Sprite()
        self.sky.loadimage('backdrops/sky.png')
        self.world.append(self.sky)
        self.peoples_entrance += 1

        #add ship, buildings
        if loc==0:
            self.ship = Sprite()
            self.ship.loadimage('ships/ship1.png')
            self.ship.pos = [100,0]
            self.world.append(self.ship)
            self.peoples_entrance += 1
        #remember spot
        
        #go through people and add ones on this screen
        
        #add foreground
        #add menu
        #add mouse again
        self.world.append(self.cursor)
        #Add interface
        self.oxygen_g = Text()
        self.oxygen_g.updatetext('O2 = ')
        self.oxygen_g.loadimage('gui/button_1.png')
        self.world.append(self.oxygen_g)
